package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.persistence.Version;

/**
 * Hibernateエンティティ共通カラムフィールドインタフェースを提供します。<br>
 * <p>
 * Hibernateアドオンモジュールで利用するエンティティはすべてこのクラスが継承されて提供されます。<br>
 * ここではHibernateを利用したデータベーステーブル操作時に最低限必要となるレコードID、レコードバージョン番号カラムのみが提供されます。<br>
 * 通常のアプリケーションで利用されるエンティティはこれを直接継承せず{@link AbstractHibernateDefaultEntity}を継承して作成されます。<br>
 * 例外的に当エンティティが継承されて作成されるエンティティは予めアドオンが提供する{@link org.phosphoresce.webcore.ext.hibernate4.model.SCode}、{@link org.phosphoresce.webcore.ext.hibernate4.model.SConfig}となります。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
@MappedSuperclass
public abstract class AbstractHibernateEntity implements Serializable {

	/** レコードID */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(name = "id", unique = true, nullable = false)
	private Long id;

	/** バージョン番号 */
	@Version
	@Column(name = "version_lock", nullable = false)
	private Integer versionLocked;

	/**
	 * データベース登録処理前のエンティティ内容の整合処理を実施します。<br>
	 */
	void validate() {
		validateEntity();
	}

	/**
	 * データベース登録処理前のエンティティ内容の整合処理を実施します。<br>
	 * 当処理はエンティティ毎に処理が異なります。<br>
	 */
	protected abstract void validateEntity();

	/**
	 * レコードIDを取得します。<br>
	 * @return レコードID
	 */
	public final Long getId() {
		return id;
	}

	/**
	 * レコードIDを設定します。<br>
	 * @param id レコードID
	 */
	final void setId(Long id) {
		this.id = id;
	}

	/**
	 * バージョン番号を取得します。<br>
	 * @return バージョン番号
	 */
	public final Integer getVersionLocked() {
		return versionLocked;
	}

	/**
	 * バージョン番号を設定します。<br>
	 * @param versionLocked バージョン番号
	 */
	final void setVersionLocked(Integer versionLocked) {
		this.versionLocked = versionLocked;
	}
}
